@@ -5,6 +5,7 @@ from collections import defaultdict
 from typing import *
 
 import numpy as np
+import pandas as pd
 
 from cognite.client.data_classes import Asset, AssetList, AssetUpdate, TimeSeriesList
 from cognite.client.utils._concurrency import execute_tasks_concurrently
@@ -35,6 +36,13 @@ class PowerAsset(Asset):
         """Safe way of getting a metadata value, returns None if metadata or key does not exist"""
         return (self.metadata or {}).get(key)
 
+    def float_metadata_value(self, key) -> Optional[float]:
+        val = self.metadata_value(key)
+        if val is not None:
+            return float(val)
+        else:
+            return None
+
     @property
     def type(self):
         """Shortcut for it's metadata-defined type (e.g. 'TransformerEnd')"""
@@ -43,11 +51,7 @@ class PowerAsset(Asset):
     @property
     def base_voltage(self) -> Optional[float]:
         """Gets the asset's base voltage as a float, or None if it has no base voltage"""
-        bv = self.metadata_value("BaseVoltage_nominalVoltage")
-        if bv is not None:
-            return float(bv)
-        else:
-            return None
+        return self.float_metadata_value("BaseVoltage_nominalVoltage")
 
     @property
     def grid_type(self) -> Optional[str]:
@@ -144,6 +148,54 @@ class BusbarSection(PowerAsset):
     pass
 
 
+class CurrentLimit(PowerAsset):
+    @property
+    def value(self) -> Optional[float]:
+        return self.float_metadata_value("CurrentLimit.value")
+
+    @property
+    def limit_type(self) -> str:
+        return self.name.split("@")[0]
+
+
+class TemperatureCurve(PowerAsset):
+    pass
+
+
+class TemperatureCurveDependentLimit(PowerAsset):
+    pass
+
+
+class TemperatureCurveData(PowerAsset):
+    @property
+    def percent(self) -> Optional[float]:
+        return self.float_metadata_value("TemperatureCurveData.percent")
+
+    @property
+    def temperature(self) -> Optional[float]:
+        return self.float_metadata_value("TemperatureCurveData.temperature")
+
+
+class OperationalLimitSet(PowerAsset):
+    pass
+
+
+class OperationalLimitType(PowerAsset):
+    pass
+
+
+class ShuntCompensator(PowerAsset):
+    pass
+
+
+class StaticVarCompensator(PowerAsset):
+    pass
+
+
+class PetersenCoil(PowerAsset):
+    pass
+
+
 class PowerTransformer(PowerAsset):
     def power_transformer_ends(
         self, end_number: Optional[Union[int, Iterable]] = None, base_voltage: Iterable = None
@@ -331,6 +383,76 @@ class ACLineSegment(PowerAsset):
         """Shortcut for finding the connected substations"""
         return self.terminals().substations()
 
+    def operational_limit_sets(self) -> "PowerAssetList":
+        return self.relationship_sources("OperationalLimitSet")
+
+    def current_limits(self) -> "PowerAssetList":
+        return self.operational_limit_sets().relationship_sources("CurrentLimit")
+
+    def temperature_curve_dependent_limits(self) -> "PowerAssetList":
+        return self.relationship_sources("TemperatureCurveDependentLimit")
+
+    def temperature_curves(self) -> "PowerAssetList":
+        return self.temperature_curve_dependent_limits().relationship_sources("TemperatureCurve")
+
+    def temperature_curve_data(self) -> "PowerAssetList":
+        return self.temperature_curves().relationship_sources("TemperatureCurveData")
+
+    def current_limits_overview(self) -> "pd.DataFrame":
+        """Returns a dataframe which combines data from `temperature_curve_data` and `current_limits` """
+        current_limits = self.current_limits()
+        temp_coefficients = self.temperature_curve_data()
+        return pd.DataFrame(
+            {
+                "type": current_limit.limit_type,
+                "temperature": coeff.temperature,
+                "limit": current_limit.value * coeff.percent,
+                "name": self.name,
+            }
+            for current_limit in current_limits
+            for coeff in temp_coefficients
+        ).set_index(["name", "type", "temperature"])
+
+    def load_duration_curve(
+        self,
+        start,
+        end="now",
+        terminal=1,
+        measurement_type="ThreePhaseActivePower",
+        timeseries_type="estimated_value",
+        granularity="1h",
+        dropna=True,
+        index_granularity=0.1,
+    ) -> "pd.DataFrame":
+        """Calculates a load-duration curve.
+
+        Args:
+            start, end: string, timestamp or datetime for start and end, as in datapoints.retrieve
+            terminal, measurement_type, timeseries_type: which measurement of which terminal to retrieve.
+            granularity: granularity to be used in retrieving time series data.
+            dropna: whether to drop NaN values / gaps
+            index_granularity: spacing of the regularized return value in %, e.g. 0.1 gives back 1001 elements.
+
+        Returns:
+            pd.DataFrame: dataframe with a load duration curve
+        """
+
+        ts = assert_single_result(
+            self.terminals(sequence_number=terminal).time_series(
+                measurement_type=measurement_type, timeseries_type=timeseries_type
+            )
+        )
+        df = self._cognite_client.datapoints.retrieve_dataframe(
+            id=ts.id, start=start, end=end, granularity=granularity, aggregates=["interpolation"], complete="fill"
+        )
+        if dropna:
+            df = df.dropna()
+        index = np.linspace(start=0, stop=1, num=round(100 / index_granularity + 1))
+        regular_spacing = np.interp(
+            index, np.linspace(start=0, stop=1, num=df.shape[0]), df.iloc[:, 0].sort_values(ascending=False).values
+        )
+        return pd.DataFrame(index=index, data=regular_spacing, columns=[self.name])
+
 
 class PowerAssetList(AssetList):
     _RESOURCE = PowerAsset
@@ -720,3 +842,38 @@ class PowerAssetList(AssetList):
         else:
             returned_assets = visited_substations
         return PowerAssetList(list(returned_assets), cognite_client=self._cognite_client)
+
+    def current_limits_overview(self) -> "pd.DataFrame":
+        """See ACLineSegment#current_limits_overview"""
+        if not self.has_type("ACLineSegment"):
+            raise WrongPowerTypeError(f"Can't get connected current limits dataframe for a list of {self.type}")
+        res_list = execute_tasks_concurrently(
+            ACLineSegment.current_limits_overview, [(a,) for a in self], max_workers=10
+        )
+        return pd.concat(res_list.joined_results())
+
+    def load_duration_curve(
+        self,
+        start,
+        end="now",
+        terminal=1,
+        measurement_type="ThreePhaseActivePower",
+        timeseries_type="estimated_value",
+        granularity="1h",
+        dropna=True,
+        index_granularity=0.1,
+    ) -> "pd.DataFrame":
+        """See ACLineSegment#load_duration_curve"""
+        if not self.has_type("ACLineSegment"):
+            raise WrongPowerTypeError(f"Can't get connected current limits dataframe for a list of {self.type}")
+        if len(self.data) > 1000:
+            raise ValueError("Too many line segments in this list to get load duration curves")
+        res_list = execute_tasks_concurrently(
+            ACLineSegment.load_duration_curve,
+            [
+                (a, start, end, terminal, measurement_type, timeseries_type, granularity, dropna, index_granularity)
+                for a in self
+            ],
+            max_workers=10,
+        )
+        return pd.concat(res_list.joined_results(), axis=1)
